package cn.estsh.i3plus.core.apiservice.auth;

import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import com.alibaba.fastjson.JSONObject;
import org.apache.shiro.cache.Cache;
import org.apache.shiro.cache.CacheManager;
import org.apache.shiro.cache.ehcache.EhCacheManager;
import org.apache.shiro.session.Session;
import org.apache.shiro.session.UnknownSessionException;
import org.apache.shiro.session.mgt.eis.EnterpriseCacheSessionDAO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/**
 * @Description : redisshiro保存会话
 * @Reference : 使用了ehcache做本地缓存，避免频繁读写redis，进行序列化和反序列化。
 *  但需要注意，本地缓存在微服的情况下可能会出现脏数据。建议在登陆时一次性放入，后续若修改则需要同时更新缓存。
 * @Author : alwaysfrin
 * @CreateDate : 2018-10-16 10:28
 * @Modify:
 **/
public class ShiroSessionRedisDaoImpl extends EnterpriseCacheSessionDAO {
    public static final Logger LOGGER = LoggerFactory.getLogger(ShiroSessionRedisDaoImpl.class);
    public static final int EXPIRE_TIME = 30 * 60; //会话保存时间 单位秒

    public static final String LOCAL_SESSION_CACHE = "redis-shiro-session-cache";

    /**
     * 缓存
     */
    @Resource(name="redisSession")
    private ImppRedis redisSession;

    //本地缓存
    private Cache localCache;

    private CacheManager localCacheManager;
    public void setLocalCacheManager(CacheManager localCacheManager) {
        this.localCacheManager = localCacheManager;
        this.localCache = localCacheManager.getCache(LOCAL_SESSION_CACHE);
    }

    @Override
    public void update(Session session) throws UnknownSessionException {
        LOGGER.info("【更新缓存 : {}】{}",session,session.getId());
        this.saveSession(session);
    }

    private void saveSession(Session session) {
        if (session == null) {
            LOGGER.error("【要存入的session为空】");
        }else {
            //LOGGER.info("【保存session至redis,id:{}】{}",session.getId(),session);
            //设置会话过期时间
            redisSession.putObject(session.getId().toString(),session,EXPIRE_TIME);

            //放入本地缓存
            localCache.put(session.getId(),session);
            LOGGER.info("【保存session至redis,id:{}】{}",session.getId(),session);
        }
    }

    @Override
    protected Session getCachedSession(Serializable sessionId) {
        //先从本地获取，若本地没有，再从redis中获取
        Session s = (Session) localCache.get(sessionId);
        if(s == null) {
            s = (Session) redisSession.getObject(sessionId.toString());

            LOGGER.info("【本地缓存不存在，redis获取：{}】{}", sessionId, s);
            if(s != null) {
                //放入本地缓存
                localCache.put(sessionId, s);
                LOGGER.info("【放入本地缓存：{}】{}", sessionId, s);
            }
        }else{
            //LOGGER.info("【本地缓存获取：{}】{}", sessionId, s);
        }

        return s;
    }

    @Override
    public void delete(Session session) {
        if (session == null) {
        }else {
            LOGGER.error("【删除session-redis：{}】",session.getId());
            //redis删除
            redisSession.deleteKey(session.getId().toString());
            //本地缓存删除
            localCache.remove(session.getId());
        }
    }

    @Override
    protected Serializable doCreate(Session session) {
        LOGGER.info("【doCreate并保存：{}】",session);
        Serializable sessionId = this.generateSessionId(session);
        this.assignSessionId(session, sessionId);
        this.saveSession(session);

        return sessionId;
    }

    @Override
    protected Session doReadSession(Serializable sessionId) {
        Session session = null;
        if(sessionId != null) {
            //先从本地缓存读取
            Object obj = localCache.get(sessionId);

            if(obj == null) {
                LOGGER.info("【doReadSession-本地缓存不存在不存在：{}】", sessionId);
                obj = redisSession.getObject(sessionId.toString());
                if (obj == null) {
                    LOGGER.info("【doReadSession不存在：{}】", sessionId);
                } else {
                    LOGGER.info("【doReadSession：{}】", sessionId);
                    session =  (Session) obj;
                }
            }else{
                LOGGER.info("【doReadSession-本地缓存获取：{}】", sessionId);
                session =  (Session) obj;
            }
        }

        return session;
    }

    /**
     * 获取存活的sessions
     */
    @Override
    public Collection<Session> getActiveSessions() {
        Set<Session> sessions = new HashSet<>();
        Set<String> keys = redisSession.getKeysSet("*");
        LOGGER.info("【所有活动的key：{}】",keys.size());
        for(String key:keys){
            sessions.add((Session)redisSession.getObject(key));
        }
        return sessions;
    }
}
